
/*
 * Copyright 2000-2011 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
* Generic popup component, which supports everything   ;))
* */
BS.Popup = function(elementId, options) {
  this._name = elementId;
  options = options || {};
  var user_shift = options.shift || {};
  delete options.shift;

  this.options = Object.extend({

    delay: 300,    // msecs before popup is shown
    hideDelay: 10,  // msecs before popup is hidden on mouse out; -1 to disable hiding
    width: null,   // if not null, corresponding width is set to the popup

    // location options, depend on how do you show popup, either
    // with showPopup or showPopupNearElement
    shift: {
      x: -200,
      y: 15
    },

    loadingText: "Loading...",

    // set to true if popup window heigh should be limited by the current window height
    fitWithinWindowHeight: false,
    fitWithinWindowBottomMargin: 0, // margin under the bottom part of the popup
    fitWithinWindowTopMargin: 5, // margin above the bottom part of the popup (we'll scroll the page)
    doScroll: true, //set to false if you do not need scroll on open popup

    // Content options. If none provided, current content of the element is shown in popup
    // Priority: innerText, url, textProvider
    innerText: null,
    url: null,
    parameters: null,
    textProvider: null, // function(popup) { return "<b> some html </b"; },

    // Function is called after load of the content
    afterShowFunc: function() { },

    // Function is called after reald hiding the popup
    afterHideFunc: function() { }

  }, options);

  Object.extend(this.options.shift, user_shift);
  this._timer = null;
};

Object.extend(BS.Popup.prototype, {

  element: function() { return $(this._name) },

  // Show this popup near mouse even location
  showPopup: function(event, moreOptions) {
    Object.extend(this.options, moreOptions || {});

    var x = Event.pointerX(event) + this.options.shift.x;
    var y = Event.pointerY(event) + this.options.shift.y;

    this._showWithDelay(x, y);
    return this;
  },

  // Show this popup near given element
  showPopupNearElement: function(element, moreOptions) {
    Object.extend(this.options, moreOptions || {});

    element = $(element);
    var pos = BS.getPosition(element);

    var x = pos[0] + this.options.shift.x;
    var y = pos[1] + this.options.shift.y;

    this._showWithDelay(x, y);
    return this;
  },

  isLoaded: function() {
    return this._loaded === undefined || this._loaded;
  },

  hidePopup: function(delay, force) {
    this._clearPopupTimer();
    if (delay === undefined) delay = this.options.hideDelay;

    if (this.isShowing() && (this.isLoaded() || force)) {
      if (delay > 0) {
        BS.Hider.startHidingDiv(this._name, delay);
      }
      else {
        BS.Hider.stopHidingDiv(this._name);
        BS.Hider.hideDiv(this._name);
      }
    }
  },

  stopHidingPopup: function() {
    if (this.isShowing()) {
      BS.Hider.stopHidingDiv(this._name);
    }
  },

  isShowing: function() {
    return BS.Util.visible(this.element());
  },

  _getMaxHeight: function() {
    if (this.options.fitWithinWindowHeight) {
      var popupPos = Element.cumulativeOffset(this.element());
      var height = BS.Util.windowSize()[1] - this.options.fitWithinWindowBottomMargin;
      height = height - (popupPos[1] - BS.Util._scrollTop());
      return height > 0 ? height: 100;
    }

    return -1;
  },

  _showWithDelay: function(x, y) {

    if (this.isShowing() && !this.isLoaded()) return;

    var delay = this.options.delay;
    this._clearPopupTimer();

    if (delay == 0) {
      this._showPopupNow(x, y);
    } else {
      this._timer = setTimeout(
        function() {
          this._showPopupNow(x, y);
        }.bind(this)
      , delay);
    }
  },

  _clearPopupTimer: function() {
    if (this._timer) {
      clearTimeout(this._timer);
      delete this._timer;
    }
  },

  _showPopupNow: function(x, y) {
    var div = this.element();
    if (!div) {
      div = document.createElement("div");
      div.id = this._name;
      div.className = 'popupDiv';
      document.body.appendChild(div);
    }
    else {
      if (div.parentNode != document.body) {
        div.remove();
        document.body.appendChild(div);
      }
    }
    if (this.options.className) {
      $(div).addClassName(this.options.className);
    }

    this._oldBackground = Element.getStyle(div, "backgroundColor");

    if (this.options.innerText) {
      div.innerHTML = this.options.innerText;
    }
    else if (this.options.url || this.options.textProvider) {
      div.style.backgroundColor="#fff";
      div.innerHTML = BS.loadingIcon + this.options.loadingText;
    }

    if (this.options.beforeShow) {
      this.options.beforeShow(div);
    }
    BS.Util.place(div, x, y);

    if (this._getMaxHeight() != -1) {
      div.style.overflow = 'hidden';
      div.style.height = 'auto';
    }

    this._loaded = false;
    BS.Hider.showDivWithTimeout(div.id, Object.extend({

      hideOnMouseOut: function() {

        return this.options.hideDelay >= 0 && this._loaded;

      }.bind(this)
    }, this.options));

    if (this.options.url){
      BS.ajaxUpdater(this._name, this.options.url, {
        method: 'get',
        evalScripts: true,
        parameters: this.options.parameters,
        onComplete: function() {
          this._onComplete();
        }.bind(this)
      });
    }
    else {
      if (this.options.textProvider) {
        div.innerHTML = this.options.textProvider(this);
      }
      this._onComplete();
    }
  },

  _onComplete: function() {
    this.element().style.backgroundColor = this._oldBackground;
    if (this.options.width) {
      this.element().style.width = this.options.width;
    }
    
    this._fixHeight();
    this.updatePopup();
    this.options.afterShowFunc(this);
    this._loaded = true;
  },

  _fixHeight: function() {
    var container = this.element();
    var maxHeight = this._getMaxHeight(container);
    if (maxHeight > 0 && container.offsetHeight > maxHeight) {
      if (this.options.doScroll) {
        window.scrollTo(0, container.cumulativeOffset()[1] - this.options.fitWithinWindowTopMargin);
      }
      container.style.height = this._getMaxHeight(container) + 'px';
      container.style.overflowY = "auto";
    }
  },

  // ensures popup is fitting the page and updates popup iframe if needed
  // should be called each time after popup content changes
  updatePopup: function() {
    BS.Util.shiftToFitPage(this.element());
    BS.Util.moveDialogIFrame(this.element());
  }
});



//-------------------------------------------------------------------------
//  Some Specific popups:



BS.ChangesPopup = new BS.Popup("changesPopup", {
  url: base_uri+"/changesPopup.html?limit=25",
  method: "get",
  hideDelay: 300
});

BS.ChangesPopup._switchTab = function(container, promoId, tab) {
  container.style.overflow = 'hidden';
  container.style.height = 'auto';
  container.innerHTML = BS.loadingIcon + "Loading...";
  this.updatePopup();
  var that = this;

  setTimeout(function() {
    new BS.ajaxUpdater(container, base_uri+"/changesPopupTab.html", {
      method: 'get',
      evalScripts: true,
      parameters: "limit=25&promoId=" + promoId + "&tab=" + tab,
      onComplete: function() {
        that.updatePopup();
      }
    });
  }, 500);
};

BS.ChangesPopup.showCurrentBuild = function(container, promoId) {
  this._switchTab(container, promoId, "");
};

BS.ChangesPopup.showSinceLastSuccessful = function(container, promoId) {
  this._switchTab(container, promoId, "sinceLastSuccessful");
};

BS.ChangesPopup.showBuildChangesPopup = function(nearestElement, promoId, tab) {
  this.options.parameters = "promoId=" + promoId + (tab != null ? "&tab=" + tab : "");
  this.showPopupNearElement(nearestElement);
};

BS.ChangesPopup.showPendingChangesPopup = function(nearestElement, buildTypeId) {
  this.options.parameters = "buildTypeId=" + buildTypeId;
  this.showPopupNearElement(nearestElement);
};

BS.FilesPopup = new BS.Popup("filesPopup", {
  url: base_uri+"/filesPopup.html",
  shift: {x: -300, y: 10},
  hideDelay: 200
});


BS.BuildResultsSummary = function(buildId, buildTypeId, finished, skipChangesArtifacts) {
  BS.Popup.call(this, "popupRes" + buildId, {
    buildId: buildId,
    skipChangesArtifacts: skipChangesArtifacts,
    buildTypeId: buildTypeId,

    shift: {x : -80},
    hideDelay: 300,

    textProvider: function(popup) {
      if (!$('buildResultsSummaryTemplate')) return "";
      var text = $('buildResultsSummaryTemplate').innerHTML;
      text = text.replace(/##BUILD_ID##/g, popup.options.buildId);
      text = text.replace(/##BUILD_TYPE_ID##/g, popup.options.buildTypeId);

      setTimeout(function() { this.showDetailedInfo(); }.bind(popup), 20);
      return text;
    }
  })
};

Object.extend(BS.BuildResultsSummary.prototype, BS.Popup.prototype);

BS.BuildResultsSummary.prototype.showDetailedInfo = function() {
  this._loaded = false;
  var buildId = this.options.buildId;

  $('summaryProgress:' + buildId).innerHTML = BS.loadingIcon + "Loading details ...";
  BS.ajaxUpdater($('detailedSummary:' + buildId), base_uri+"/buildResultsSummary.html?buildId=" +
                                                      buildId + "&skipChangesArtifacts=" + this.options.skipChangesArtifacts, {
    evalScripts: true,
    onComplete: function() {
      if ($('summaryProgress:' + buildId))
        $('summaryProgress:' + buildId).innerHTML = "";
      this.updatePopup();
      this._loaded = true;
    }.bind(this)
  });
};


BS.BuildTypeSummary = new BS.Popup("buildTypeSummary", {
  shift: {x : 5, y: 20},
  hideDelay: 300,

  textProvider: function(popup) {
    if (!$('buildTypeMenuTemplate')) return "";
    var text = $('buildTypeMenuTemplate').innerHTML;
    text = text.replace(/##BUILD_TYPE_ID##/g, popup.options.buildTypeId);
    text = text.replace(/##BUILD_TYPE_NAME##/g, popup.options.buildTypeName.replace(/['"]/, '`'));
    text = text.replace(/##PROJECT_ID##/g, popup.options.projectId);

    if (!popup.options.withSelf) {
      text = text.replace(/self_element="true"/g, "style='display:none'");
    }
    if (!popup.options.withAdmin) {
      text = text.replace(/admin_element="true"/g, "style='display:none'");
    }
    if (!popup.options.showResponsibility) {
      text = text.replace(/responsibility="true"/g, "style='display:none'");
    }

    return text;
  }
});
BS.BuildTypeSummary.showSummaryPopup = function(nearestElement, buildTypeId, projectId, withSelf, withAdmin, showResponsibility, buildTypeName) {
  this.options.buildTypeId = buildTypeId;
  this.options.buildTypeName = buildTypeName;
  this.options.projectId = projectId;
  this.options.withSelf = withSelf;
  this.options.withAdmin = withAdmin;
  this.options.showResponsibility = showResponsibility;
  this.showPopupNearElement(nearestElement);
};


BS.AgentInfoPopup = new BS.Popup("agentInfoPopup", {
  url: base_uri+"/showCompatibleAgents.html",
  shift: {x: -150}
});
BS.AgentInfoPopup.showAgentsPopup = function(nearestElement, itemId) {
  this.options.parameters = "itemId=" + itemId;
  this.showPopupNearElement(nearestElement);
};

BS.ArtifactsPopup = new BS.Popup("artifactsPopup", {
  shift: {x: -60},
  url: base_uri+"/viewArtifactsPopup.html",
  hideDelay: 300
});
BS.ArtifactsPopup.showArtifactsPopup = function(nearestElement, buildTypeId, buildId) {
  BS.ArtifactsPopup.options.parameters = "buildId=" + buildId;
  BS.ArtifactsPopup.showPopupNearElement(nearestElement);
};



BS.BuildTypeSettingsPopup = {};
BS.BuildTypeSettingsPopup.showMultilineValue = function(element, title, text) {
  BS.Util.center($('valuePopup'));
  $('valueContainer').value = text;
  $('valuePopupTitle').innerHTML = title;
  BS.Hider.showDivWithTimeout('valuePopup', {
    hideOnMouseOut: false,
    draggable: true,
    dragHandle: $('valuePopupHeader')
  });

  $('valueContainer').focus();
};
BS.BuildTypeSettingsPopup.closeMultilineValuePopup = function() {
  BS.Hider.hideDiv('valuePopup');
};

BS.ProgressPopup = new BS.Popup("progressPopup", {
  delay: 0,
  hideDelay: 0
});
BS.ProgressPopup.showProgress = function(nearestElem, message, options) {
  this.options.innerText = "<nobr>" + BS.loadingIcon + message + "</nobr>";
  if (this.options.delay === undefined) {
    this.options.delay = 0;
  }

  Object.extend(this.options, options);
  this.showPopupNearElement(nearestElem);
};

BS.WarningPopup = new BS.Popup("warningPopup", {
  delay: 0,
  hideOnMouseOut: false
});
BS.WarningPopup.showWarning = function(nearestElem, shift, message, moreOptions) {
  this.options.innerText = message;
  this.options.shift = shift;
  setTimeout(function() {
    this.showPopupNearElement(nearestElem, moreOptions)
  }.bind(this), 50)
};

/* simple popup for displaying messages */
BS.Tooltip = {
  _id: 1,

  getId: function() {
    return "messagePopup_" + this._id++;
  }
};

BS.Tooltip.create = function(options, message) {
  BS.Tooltip.popup = new BS.Popup(BS.Tooltip.getId(), Object.extend({
    delay: options.delay||400,
    shift: options.shift,
    hideOnMouseOut: true,
    hideOnMouseClickOutside: false,
    innerText: message
  }, options));
  return BS.Tooltip.popup;
};
BS.Tooltip.showMessage = function(nearestElem, options, message) {
  BS.Tooltip.create(options, message).showPopupNearElement(nearestElem);
};
BS.Tooltip.showMessageFromContainer = function(nearestElem, options, container) {
  BS.Tooltip.showMessage(nearestElem, options, $(container).innerHTML);
};
BS.Tooltip.showMessageAtCursor = function(event, options, message) {
  BS.Tooltip.create(options, message).showPopup(event);
};
BS.Tooltip.hidePopup = function(t, t1) {
  if (BS.Tooltip.popup) {
    BS.Tooltip.popup.hidePopup(t, t1);
  }
};

BS.QueuedBuildsPopup = new BS.Popup("queuedBuildsPopup", {
  url: base_uri+"/queue.html",
  method: "get"
});

BS.QueuedBuildsPopup.showQueuedBuilds = function(nearestElem, buildTypeId) {
  this.showPopupNearElement(nearestElem, {parameters: "forBuildTypeId=" + buildTypeId, shift: {x: -550, y: 15}});
};

BS.ClickPopupSupport = {
  clickPopups: {},

  togglePopup: function(id, showPopupCommand) {
    var popup = BS.ClickPopupSupport.clickPopups[id];
    if (!popup || !popup.isShowing()) {
      popup = eval(showPopupCommand);
      BS.ClickPopupSupport.clickPopups[id] = popup;
      $(this).addClassName('actionLinkNewHover');

      var linkPos = Position.cumulativeOffset($(this));
      var linkDim = $(this).getDimensions();
      var elDim = popup.element().getDimensions();

      BS.Util.place(popup.element(), linkPos[0] + linkDim.width - elDim.width, linkPos[1] + linkDim.height - 1);
      popup.updatePopup();

      BS.Hider.addHideFuntion(popup._name, function() {
        $(this).removeClassName('actionLinkNewHover');
        delete BS.ClickPopupSupport.clickPopups[id];
      }.bind(this));
    }
    else {
      popup.hidePopup(0, true);
    }
  }
};

BS.PromoDetailsPopup = new BS.Popup("promoDetailsPopup", {
  url: base_uri + "/promoDetailsPopup.html",
  method: "get",
  shift: {x : -50}
});

BS.PromoDetailsPopup.showDetailsPopup = function(nearestElement, promoId) {
  BS.PromoDetailsPopup.options.parameters = "promoId=" + promoId;
  BS.PromoDetailsPopup.showPopupNearElement(nearestElement);
};

BS.AuthorityRolesPopup = {};
BS.AuthorityRolesPopup = new BS.Popup("authorityRolesPopup", {
  url: base_uri+"/admin/authorityRolesPopup.html",
  method: "get"
});

BS.AuthorityRolesPopup.showPopup = function(nearestElement, userId) {
  this.options.parameters = "rolesHolderId=" + userId;
  this.showPopupNearElement(nearestElement);
};

BS.GroupUsersPopup = {};
BS.GroupUsersPopup = new BS.Popup("groupUsersPopup", {
  url: base_uri+"/admin/groupUsersPopup.html",
  method: "get"
});

BS.GroupUsersPopup.showPopup = function(nearestElement, groupCode) {
  this.options.parameters = "groupCode=" + encodeURIComponent(groupCode);
  this.showPopupNearElement(nearestElement);
};

BS.ParentGroupsPopup = {};
BS.ParentGroupsPopup = new BS.Popup("parentGroupsPopup", {
  url: base_uri+"/admin/parentGroupsPopup.html",
  method: "get"
});

BS.ParentGroupsPopup.showUserParentGroups = function(nearestElement, userId) {
  this.options.parameters = "userId=" + userId;
  this.showPopupNearElement(nearestElement);
};

BS.ParentGroupsPopup.showGroupParentGroups = function(nearestElement, groupCode) {
  this.options.parameters = "groupCode=" + encodeURIComponent(groupCode);
  this.showPopupNearElement(nearestElement);
};

BS.TemplateUsagesPopup = new BS.Popup("templateUsagesPopup", {
  url: base_uri + "/admin/templateUsagesPopup.html",
  method: "get"
});

BS.TemplateUsagesPopup.showPopup = function(nearestElement, templateId, selectedStep) {
  this.options.parameters = "templateId=" + encodeURIComponent(templateId) + "&selectedStep=" + selectedStep;
  this.showPopupNearElement(nearestElement);
};

BS.ShowBuildTypesPopup = new BS.Popup('showBuildTypesPopup', {
  delay: 300,
  hideDelay: 0,
  shift: {x: -230, y: 20},
  className: 'hiddenBuildTypes'
});
BS.ShowBuildTypesPopup.showPopup = function(nearestElement, projectId) {
  this.options.url = base_uri + '/showBuildTupesPopup.html?projectId=' + projectId;
  this.showPopupNearElement(nearestElement);
};

BS.TogglePopup = new BS.Popup('togglePopup', {
  delay: 0,
  hideDelay: -1,
  shift: {x: 0}
});
BS.TogglePopup.toggle = function(link) {
  if (this.isShowing()) {
    this.hidePopup();
  } else {
    this.options.innerText = link.nextSibling.innerHTML;
    this.showPopupNearElement(link);
  }
  return false;
};


/*===========================================================================*/
/*   Handler for simplePopup.tag     */
/*===========================================================================*/
BS.install_simple_popup = function(elementId, options) {
  options = options || {};

  var el = $(elementId);
  if (!el) {
    BS.Log.warn("install_simple_popup: Cannot find element with id " + elementId);
    return;
  }

  var popup = new BS.Popup(elementId + "Content", options);

  el.on("mouseover", function(e) {
    popup.stopHidingPopup();
  });

  el.on("mouseover", "img", function(e, img) {
    popup.showPopupNearElement(img);
    img.src = img.src.replace('.gif', '_hover.gif');
  });

  el.on("mouseout", "img", function(e, img) {
    popup.hidePopup(300);
    img.src = img.src.replace('_hover', '');
  });

  el.on("click", "img", function(e, img) {
    Event.stop(e);
  });

  var link = el.down(".popupLink");
  if (link) {
    link.on("click", function(e, ahref) {
      popup.showPopupNearElement(ahref, {delay: 0});
    });
  }

};


